package fr.free.supertos;

import java.util.Random;
import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;


public class FloconsActivity extends Activity {
	
	private ProgressDialog progressDialog;
	Thread t;
	RenderView renderView;
	
	// Classe interne de gestion des messages en provenance des 2 threads
    Handler handlerMsg= new Handler()
    {
		 
    	public void handleMessage(android.os.Message msg)
    	{
    		// Fin du thread de temporisation sur l'affichage dialogue
    		if(msg.what == 0) {
    			progressDialog.dismiss();
    		}
    		else
    			// Fin du thread de temporisation pour l'animation
        		if(msg.what == 1) {
        			renderView.invalidate();
        		}

   		}
   	};

    @Override
    public void onCreate(Bundle savedInstanceState) {
    	
    	Log.i("flocons", "onCreate");
        super.onCreate(savedInstanceState);
                       
        // Desactiver la barre de titre de notre application
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        // Passer la fentre en full-creen == cacher la barre de notifications
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN,
        WindowManager.LayoutParams.FLAG_FULLSCREEN);

        progressDialog = new ProgressDialog(this);
        
        progressDialog.setMessage("Flocons de neige !");
        // On affiche notre message
        progressDialog.show();
        // Lancement du thread de temporisation affichage dialogue
        t = new MonThread(handlerMsg);
        t.start();
    	renderView = new RenderView(this);
    	setContentView(renderView);
    }
    
    @Override
    public void onResume()
    {
    	Log.i("flocons", "onResume");
    	// Cration ou recrtion d'une instance pour le rendu 2D

    	super.onResume();
    	
    	renderView.onResume();
    }

    @Override
    public void onPause()
    {
    	Log.i("flocons", "onPause");
    	// Si le thread de tempo dialogue existe encore
    	if (t.isAlive())
    	{
    		// Le flinguer
    		t.interrupt();
    		// Et fermer la dialogue
    		progressDialog.dismiss();
    	}
    	
    	// Mme chose pour le thread de temporisation animation dans le render
    	renderView.onPause();	 	
	 	
    	super.onPause();
    }

    // Instance de thread permettant de temporiser l'affichage de notre dialogue  5 secondes
    public class MonThread extends Thread {
    	Handler handlerDial;
        public MonThread(Handler handlerDial) {
			// TODO Auto-generated constructor stub
        	this.handlerDial=handlerDial;
		}

		public void run() {
            // Traitement
            try {
            	Thread.sleep(5000);
            	handlerDial.sendEmptyMessage(0);
            } catch (InterruptedException ex) {
                Thread.currentThread().interrupt(); // Marque le thread comme interrompu
                return;
            }
        }
    }
    
    // Cration d'une classe interne RenderView pour grer un affichage simple de flocons
    class RenderView extends View {

    	private final static int MAXFLOCON = 100;
    	Flocon [] flocon = new Flocon[MAXFLOCON];
    	Thread tAnim;
    	
    	private int nbFlocon=0;

    	
        public RenderView(Context context) {
            super(context);            
        }

        // Dessinons sur la totalit de l'cran
        protected void onDraw(Canvas canvas) {
        	
            canvas.drawRGB(0, 0, 0);
            Paint paint = new Paint();
            paint.setAntiAlias(true);
            
            // Nous allons dessiner nos points par rapport  la rsolution de l'cran
            int iWidth = canvas.getWidth(); // Largeur
            int iHeight = canvas.getHeight(); // Hauteur
            
            // Un flocon est blanc
            paint.setARGB(255, 255,255,255);
            
            Random rand = new Random();
            
            for (int i = 0; i< MAXFLOCON; i++)
            {
            	
            	
            	// Si pour cet lment du tableau, il n'y a pas d'instance de flocon
            	if (flocon[i] == null)
	            {
            		
            		// Nous allons limiter le nb. de flocons  10 max. via nbFlocon
            		// cela nous evitera de voir tomber tous les flocons d'un coup !
            		// De plus, pour un flocon qui pourrait tomber, nous faisons encore fonctionner l'alatoire
            		// pour voir s'il tombe effectivement !
            		int dep = rand.nextInt(100);
            		if (dep < 5 && nbFlocon < 10)
            		{
            			flocon[i] = new Flocon (iWidth, iHeight);
            			nbFlocon++;
            		}
	            }
            	else
            	{
            		// Une instance existe, donc
            		// Affichage du flocon de neige
	            	canvas.drawCircle(flocon[i].getX(), flocon[i].getY(), flocon[i].getTaille(), paint);
	            	flocon[i].tombeFlocon();// Faisons le tomber...
	            	if (flocon[i].enBas())
	            	{
	            		flocon[i] = null; // Si arriv en bas, fin de l'instance
	            	}
            	}            	
            }            
            
            // 10 Flocns max, compteur qui est lui mme soumis au hazard pour se dcrmenter !
        	if (rand.nextInt(10) < 2)
        		if (nbFlocon> 0)
        			nbFlocon--;

        }
        
        public void onPause()
        {
        	// Arrt du thread temporisation animation
        	if (tAnim.isAlive())
        		tAnim.interrupt();
        }
        
        public void onResume()
        {
            // Lancement du Thread permettant de temporiser l'animation
        	tAnim = new ThreadAnima();
        	tAnim.start();  
        }
        // Ce thread ne fait pas grand chose que dormir est signaler son rveil
        // Il servira pour l'animation afin d'avoir une animation fluide et raliste.
        public class ThreadAnima extends Thread{

        		@Override
        		public void run() {
        			while (true)
        			{
            			try {
            				Thread.sleep(100);
            				handlerMsg.sendEmptyMessage(1);
            			} catch  (InterruptedException e) {
            				Thread.currentThread().interrupt(); // Marque le thread comme interrompu
            				return; // Et on part...
            			}
        			}
        		}
        }
    }	
	
}